#include common_scripts\utility;
#include maps\mp\_utility;
#include maps\mp\perks\_perks;
#include maps\mp\gametypes\_hud_util;
#include maps\mp\mods\disableClassMenu;

// Rallypoints should be destroyed on leaving your team/getting killed
// Doesn't seem to be setting angle on spawn so that you are facing your rallypoint

/*
	Hide and Seek
	Attackers objective: Kill all enemies.
	Defenders objective: Hide as an entity on the map, optionally meleeing attackers.
	Round ends:	When one team is eliminated, or roundlength time is reached
	Map ends:	When one team reaches the score limit, or time limit or round limit is reached
	Respawning:	Players remain dead for the round and will respawn at the beginning of the next round

	For this gametype, special key bindings need to be enabled.

	- Switching into a prop(menu)
	- Switching into a prop(quick)
	- Rotating the prop(fast)
	- Rotating the prop(slow)

	Level requirements
	------------------
		Allied Spawnpoints:
			classname		mp_sd_spawn_attacker
			Allied players spawn from these. Place at least 16 of these relatively close together.

		Axis Spawnpoints:
			classname		mp_sd_spawn_defender
			Axis players spawn from these. Place at least 16 of these relatively close together.

		Spectator Spawnpoints:
			classname		mp_global_intermission
			Spectators spawn from these and intermission is viewed from these positions.
			Atleast one is required, any more and they are randomly chosen between.

	Level script requirements
	-------------------------
		Team Definitions:
			game["seekers"] = "allies";
			game["hiders"] = "axis";

*/

/*QUAKED mp_sd_spawn_attacker(0.0 1.0 0.0) (-16 -16 0) (16 16 72)
Attacking players spawn randomly at one of these positions at the beginning of a round.*/

/*QUAKED mp_sd_spawn_defender(1.0 0.0 0.0) (-16 -16 0) (16 16 72)
Defending players spawn randomly at one of these positions at the beginning of a round.*/

main()
{
	// Make gametype display properly in server browser
	setDvar("ui_gametype", "Hide & Seek");

	// We don't run this gametype in the menu
	if (getdvar("mapname") == "mp_background")
		return;

	// Define allowed seeker weapons
	level.seekerWeapons = [];
	level.seekerWeapons[0] = "m16";
	level.seekerWeapons[1] = "m4";
	level.seekerWeapons[2] = "ump45";
	level.seekerWeapons[3] = "fal";
	level.seekerWeapons[4] = "scar";
	level.seekerWeapons[5] = "ak47";
	level.seekerWeapons[6] = "p90";
	level.seekerWeapons[7] = "famas";

	// Time for hiders to hide before releasing seekers
	// TODO - make this a dvar

	// Performance tweaking
	// TODO - currently does not allow more than 200 in dvar
	if (getDvar("sys_cpughz") > 2)
	{
		setDvar("sv_network_fps", 200);
	}

	// Make DVars available
	level.gameType = "hns";
	setDvarIfUninitialized("scr_" + level.gameType + "_roundswitch", 1);
	setDvarIfUninitialized("scr_" + level.gameType + "_timelimit", 5);
	setDvarIfUninitialized("scr_" + level.gameType + "_scorelimit", 1);
	setDvarIfUninitialized("scr_" + level.gameType + "_roundlimit", 0);
	setDvarIfUninitialized("scr_" + level.gameType + "_winlimit", 4);
	setDvarIfUninitialized("scr_" + level.gameType + "_halftime", 0);
	setDvarIfUninitialized("scr_" + level.gameType + "_numlives", 1);

	maps\mp\gametypes\_globallogic::init();
	maps\mp\gametypes\_callbacksetup::SetupCallbacks();
	maps\mp\gametypes\_globallogic::SetupCallbacks();

	level.teamBased = true;

	// General game logic DVars
	registerRoundSwitchDvar(level.gameType, 1, 0, 9);
	registerTimeLimitDvar(level.gameType, 5, 0, 1440);
	registerScoreLimitDvar(level.gameType, 1, 0, 500);
	registerRoundLimitDvar(level.gameType, 0, 0, 12);
	registerWinLimitDvar(level.gameType, 4, 0, 12);
	registerHalfTimeDvar(level.gameType, 0, 0, 1);
	registerNumLivesDvar(level.gameType, 1, 0, 10);

	// Specific DVars for this gametype
	registerPrereleaseDvar(level.gameType, 30, 0, 300);

	// event handlers
	//level.onXPEvent = ::onXPEvent;

	level.getSpawnPoint = ::getSpawnPoint;
	level.onSpawnPlayer = ::onSpawnPlayer;
	level.onRespawnDelay = maps\mp\gametypes\_globallogic::blank;

	level.onTimeLimit = ::onTimeLimit;
	level.onHalfTime = maps\mp\gametypes\_gamelogic::default_onHalfTime;
	level.onDeadEvent = ::onDeadEvent;
	level.onOneLeftEvent = ::onOneLeftEvent;

	level.onPrecacheGametype = ::onPrecacheGametype;
	level.onStartGameType = ::onStartGameType;
	level.onPlayerKilled = ::onPlayerKilled;
	level.onNormalDeath = ::onNormalDeath;

	game["dialog"]["gametype"] = "hns";

	if (getDvarInt("g_hardcore"))
		game["dialog"]["gametype"] = "hc_" + game["dialog"]["gametype"];
	else if (getDvarInt("camera_thirdPerson"))
		game["dialog"]["gametype"] = "thirdp_" + game["dialog"]["gametype"];
	else if (getDvarInt("scr_diehard"))
		game["dialog"]["gametype"] = "dh_" + game["dialog"]["gametype"];
	else if (getDvarInt("scr_" + level.gameType + "_promode"))
		game["dialog"]["gametype"] = game["dialog"]["gametype"] + "_pro";

	// stretch regen time
	maps\mp\gametypes\_tweakables::registerTweakable("player", "healthregentime", "scr_player_healthregentime", 10);

	// disable all unwanted kinds of weapons
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowfrag", "scr_weapon_allowfrags", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowsmoke", "scr_weapon_allowsmoke", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowflash", "scr_weapon_allowflash", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowc4", "scr_weapon_allowc4", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowclaymores", "scr_weapon_allowclaymores", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowrpgs", "scr_weapon_allowrpgs", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("weapon", "allowmines", "scr_weapon_allowmines", 0);

	// disable all hardpoints
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowartillery", "scr_hardpoint_allowartillery", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowuav", "scr_hardpoint_allowuav", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowsupply", "scr_hardpoint_allowsupply", 0);
	maps\mp\gametypes\_tweakables::setTweakableValue("hardpoint", "allowhelicopter", "scr_hardpoint_allowhelicopter", 0);

	level disableClassMenu();
	level maps\mp\mods\_modellist::initModelList();

	initGameFlags();
	gameFlagInit("seekers_released", false);
}

onPrecacheGametype()
{
	logString("onPrecacheGametype called");

	//game["seekers_released_sound"] = "bomb_taken"; <= doesn't work
	//game["seekers_released_sound"] = "mp_obj_taken";
	game["seekers_released_sound"] = "elev_bell_ding";
	game["seekers_blind_sound"] = "elev_run_end";
	//game["hiders_changed_sound"] = "claymore_activated";
	game["hiders_changed_sound"] = "weap_c4detpack_trigger_plr";
	//game["hiders_changed_sound"] = "positions_lock";

	game["fx_confetti"] = LoadFX("props/cash_player_drop");
	game["fx_boom"] = LoadFX("props/electricbox4_explode");

	PreCacheString(&"HIDEANDSEEK_TIMER_SEEKER_RELEASE");
	PreCacheString(&"HIDEANDSEEK_TIMER_SEEKER_RELEASE_TITLE");
	PreCacheString(&"HIDEANDSEEK_SEEKERS_RELEASED");
	PreCacheString(&"OBJECTIVES_HIDEANDSEEK_DEFENDER");
}

registerPrereleaseDvar(dvarString, defaultValue, minValue, maxValue)
{
	logString("registerPrereleaseDvar called");

	registerWatchDvarInt("prereleasetime", defaultValue);

	dvarString = "scr_" + dvarString + "_prereleasetime";

	level.preReleaseTimeDvar = dvarString;
	level.preReleaseTimeMin = minValue;
	level.preReleaseTimeMax = maxValue;
	level.preReleaseTime = getDvarInt(dvarString, defaultValue);

	if (level.preReleaseTime < minValue)
	{
		level.preReleaseTime = minValue;
	}
	else if (level.preReleaseTime > maxValue)
	{
		level.preReleaseTime = maxValue;
	}
}

updateGametypeDvars()
{
	logString("updateGametypeDvars called");

	// no time <= {prereleasetime} <= 5 minutes
	level.preReleaseTime = dvarIntValue("prereleasetime", 30, 0, 5 * 60);

}

onStartGameType()
{
	logString("onStartGameType called");

	// reset flags
	gameFlagClear("seekers_released");

	if (!IsDefined(game["switchedsides"]))
		game["switchedsides"] = false;

	// Define who's seekers and who's hiders
	if (!IsDefined(game["seekers"]) || !IsDefined(game["hiders"]))
	{
		game["seekers"] = "allies";
		game["hiders"] = "axis";

		// compatibility with the game's own game logic
		game["attackers"] = game["seekers"];
		game["defenders"] = game["hiders"];
	}

	// handle side switching
	if (game["switchedsides"])
	{
		oldSeekers = game["seekers"];
		oldHiders = game["hiders"];
		game["seekers"] = oldHiders;
		game["hiders"] = oldSeekers;
	}

	// team setup
	setDvar("g_TeamName_" + game["hiders"], "Hiders");
	setDvar("g_TeamName_" + game["seekers"], "Seekers");

	// prevent spectators ghosting through nickchanging
	setClientNameMode("auto_change");

	// objective text
	// TODO
	setObjectiveText(game["seekers"], &"OBJECTIVES_WAR");
	setObjectiveText(game["hiders"], &"OBJECTIVES_HIDEANDSEEK_DEFENDER");
	if (!level.splitscreen)
	{
		setObjectiveScoreText(game["seekers"], &"OBJECTIVES_WAR_SCORE");
		setObjectiveScoreText(game["hiders"], &"OBJECTIVES_SD_DEFENDER_SCORE");
	}
	setObjectiveHintText(game["seekers"], &"OBJECTIVES_WAR_HINT");
	setObjectiveHintText(game["hiders"], &"OBJECTIVES_HIDEANDSEEK_DEFENDER");

	// spawn points
	level.spawnMins = (0, 0, 0);
	level.spawnMaxs = (0, 0, 0);
	maps\mp\gametypes\_spawnlogic::placeSpawnPoints("mp_sd_spawn_attacker");
	maps\mp\gametypes\_spawnlogic::placeSpawnPoints("mp_sd_spawn_defender");

	// center of map
	level.mapCenter = maps\mp\gametypes\_spawnlogic::findBoxCenter( level.spawnMins, level.spawnMaxs );
	setMapCenter( level.mapCenter );

	// game objects on map
	allowed[0] = "dm";
	allowed[1] = "war";
	allowed[2] = level.gameType;
	maps\mp\gametypes\_gameobjects::main(allowed);

	// remove turrets
	level deletePlacedEntity("misc_turret");

	// scoring
	maps\mp\gametypes\_rank::registerScoreInfo("win", 2);
	maps\mp\gametypes\_rank::registerScoreInfo("loss", 1);
	maps\mp\gametypes\_rank::registerScoreInfo("tie", 1.5);
	//
	maps\mp\gametypes\_rank::registerScoreInfo("kill", 50);
	maps\mp\gametypes\_rank::registerScoreInfo("headshot", 50);
	maps\mp\gametypes\_rank::registerScoreInfo("assist", 20);

	level updateGametypeDvars();

	level thread levelReleaseTimer();
	level thread levelSpeeding();
}

getSpawnPoint()
{
	logString("getSpawnPoint called");

	if (self.pers["team"] == game["seekers"])
		spawnPointName = "mp_sd_spawn_attacker";
	else
		spawnPointName = "mp_sd_spawn_defender";

	spawnPoints = maps\mp\gametypes\_spawnlogic::getSpawnpointArray(spawnPointName);
	assert(spawnPoints.size);
	spawnpoint = maps\mp\gametypes\_spawnlogic::getSpawnpoint_Random(spawnPoints);

	return spawnpoint;
}

onSpawnPlayer()
{
	logString("onSpawnPlayer called");

	player = self;

	//player.spawnprotection = 0;
	//player thread DoSpawnProtection();

	//player SetClientDvar("cg_scoreboardItemHeight", 13);
	player SetClientDvar("lowAmmoWarningColor1", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningColor2", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoAmmoColor1", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoAmmoColor2", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoReloadColor1", 0, 0, 0, 0);
	player SetClientDvar("lowAmmoWarningNoReloadColor2", 0, 0, 0, 0);

	if (player.pers["team"] == game["hiders"])
	{
		player thread handleHider();
	}
	else if (player.pers["team"] == game["seekers"])
	{
		player thread handleSeeker();
	}

	level notify("spawned_player");
}

onPlayerKilled(eInflictor, attacker, iDamage, sMeansOfDeath, sWeapon, vDir, sHitLoc, psOffsetTime, deathAnimDuration, killId)
{
	logString("onPlayerKilled called");

	player = self;

	// If victim has a custom model, destroy it
	if (IsDefined(player.customModel))
	{
		player.customModel Delete();
		if (IsDefined(game["hiders_changed_sound"]))
			player PlaySound(game["hiders_changed_sound"]);
	}

	// If victim had a custom model set, reset it
	if (IsDefined(player.originalModel))
	{
		player SetModel(player.originalModel);
		if (IsDefined(game["hiders_changed_sound"]))
			player PlaySound(game["hiders_changed_sound"]);
		player.originalModel = undefined;
	}
	if (IsDefined(player.originalModelAttachments))
	{
		player DetachAll();
		for (index = 0; index < player.originalModelAttachments.size; index++)
		{
			player Attach(player.originalModelAttachments[index].model, player.originalModelAttachments[index].tag);
		}
		player.originalModelAttachments = undefined;
	}

	// Update spectator permissions
	thread checkAllowSpectating();
}


checkAllowSpectating()
{
	logString("checkAllowSpectating called");

	wait(0.05);

	if (!level.aliveCount[game["seekers"]])
	{
		level.spectateOverride[game["seekers"]].allowEnemySpectate = 1;
		maps\mp\gametypes\_spectating::updateSpectateSettings();
	}

	if (!level.aliveCount[game["hiders"]])
	{
		level.spectateOverride[game["hiders"]].allowEnemySpectate = 1;
		maps\mp\gametypes\_spectating::updateSpectateSettings();
	}
}


onDeadEvent(team)
{
	logString("onDeadEvent called");

	if (team == "all")
	{
		thread maps\mp\gametypes\_gamelogic::endGame("tie", game["strings"]["tie"]);
	}
	else if (team == game["seekers"])
	{
		level thread maps\mp\gametypes\_gamelogic::endGame(game["hiders"], game["strings"][game["seekers"]+"_eliminated"]);
	}
	else if (team == game["hiders"])
	{
		level thread maps\mp\gametypes\_gamelogic::endGame(game["seekers"], game["strings"][game["hiders"]+"_eliminated"]);
	}
}


onTimeLimit()
{
	logString("onTimeLimit called");

	// Hiders win when time limit reached
	thread maps\mp\gametypes\_gamelogic::endGame(game["hiders"], game["strings"]["time_limit_reached"]);
}


onNormalDeath(victim, attacker, lifeId)
{
	logString("onNormalDeath called");

	// Make sure the kill score is set properly
	score = maps\mp\gametypes\_rank::getScoreInfoValue("kill");
	assert(isDefined(score));

	thread maps\mp\_matchdata::logKillEvent(lifeId, victim.pers["team"]);

	// Is this the final kill?
	if (game["state"] == "postgame")
		attacker.finalKill = true;
}


onOneLeftEvent(team)
{
	logString("onOneLeftEvent called");

	player = getLastLivingPlayer(team);

	player thread giveLastOnTeamWarning();
}


giveLastOnTeamWarning()
{
	logString("giveLastOnTeamWarning called");

	player = self;

	player endon("death");
	player endon("disconnect");
	level endon("game_ended");

	player waitTillRecoveredHealth(3);

	otherTeam = getOtherTeam(player.pers["team"]);
	level thread teamPlayerCardSplash("callout_lastteammemberalive", player, player.pers["team"]);
	level thread teamPlayerCardSplash("callout_lastenemyalive", player, otherTeam);
	level notify("last_alive", player);
}

levelReleaseTimer()
{
	logString("levelReleaseTimer called");

	gameFlagClear("seekers_released");
	gameFlagWait("prematch_done");

	if (!level.preReleaseTime)
	{
		gameFlagSet("seekers_released");
		return;
	}

	maps\mp\gametypes\_gamelogic::pauseTimer();

	timerDisplays = [];

	// Display a pre-release timer on the top right corner for the hiders
	timerDisplay = createServerTimer("objective", 1.4, game["hiders"]);
	timerDisplay setPoint("TOPRIGHT", "TOPRIGHT", -8, 8);
	timerDisplay.archived = false;
	timerDisplay.hideWhenInMenu = false;
	timerDisplay.label = &"HIDEANDSEEK_TIMER_SEEKER_RELEASE";
	timerDisplay.alpha = 1;
	timerDisplay setTimer(level.preReleaseTime);
	timerDisplays[0] = timerDisplay;

	// Display a pre-release timer in the center for the seekers
	timerDisplayText = createServerFontString("objective", 1.5, game["seekers"]);
	timerDisplayText setPoint("CENTER", "CENTER", 0, -40);
	timerDisplayText.sort = 1001;
	timerDisplayText setText(&"HIDEANDSEEK_TIMER_SEEKER_RELEASE_TITLE");
	timerDisplayText.foreground = false;
	timerDisplayText.hideWhenInMenu = true;
	timerDisplays[1] = timerDisplayText;
	timerDisplay = createServerTimer("hudbig", 1, game["seekers"]);
	timerDisplay setPoint("CENTER", "CENTER", 0, 0);
	timerDisplay.sort = 1001;
	timerDisplay.color = (1,1,0);
	timerDisplay.foreground = false;
	timerDisplay.hideWhenInMenu = true;
	timerDisplay maps\mp\gametypes\_hud::fontPulseInit();
	//timerDisplay setTimer(level.preReleaseTime);
	timerDisplays[2] = timerDisplay;

	waittillframeend; // wait till cleanup of previous start timer if multiple happen at once
	timerSeconds = level.preReleaseTime;
	for (timerSeconds = level.preReleaseTime; timerSeconds > 0 && !level.gameEnded; timerSeconds--)
	{
		timerDisplays[2] thread maps\mp\gametypes\_hud::fontPulse(level);
		wait (timerDisplays[2].inFrames * 0.05);
		timerDisplays[2] SetValue(timerSeconds);
		wait (1 - (timerDisplays[2].inFrames * 0.05));
	}

	if (!level.gameEnded)
	{
		// Prerelease phase done
		gameFlagSet("seekers_released");
		maps\mp\gametypes\_gamelogic::resumeTimer();
	}

	for (index = 0; index < timerDisplays.size; index++)
		timerDisplays[index] destroyElem();
}

levelSpeeding()
{
	logString("levelSpeeding called");

	level endon("game_ended");

	originalSpeed = GetDvarInt("g_speed");
	originalSprintUnlimited = GetDvar("player_sprintUnlimited");

	thread levelRevertSpeedingWhenGameEnds(originalSpeed, originalSprintUnlimited);

	setDvar("g_speed", originalSpeed * 1.6);
	setDvar("player_sprintUnlimited", 1);

	gameFlagWait("seekers_released");

	setDvar("g_speed", originalSpeed);
	setDvar("player_sprintUnlimited", originalSprintUnlimited);
}

levelRevertSpeedingWhenGameEnds(originalSpeed, originalSprintUnlimited)
{
	logString("levelRevertSpeedingWhenGameEnds called");

	level endon("seekers_released");
	level waittill("game_ended");

	setDvar("g_speed", originalSpeed);
	setDvar("player_sprintUnlimited", originalSprintUnlimited);
}

handleSeeker() {
	logString("handleSeeker called");

	player = self;

	player endon("disconnect");
	player endon("death");
	level endon("game_ended");

	// Wait for _playerlogic to give the player his default weapons and perks
	player waittill("spawned_player");

	// Disable third-person mode
	player SetClientDvar("cg_thirdperson", 0);

	// Set player health high enough for stabbing to not instantly kill them
	//player.maxhealth = 280;
	//player.health = 280;

	// Take weapons and perks
	player ClearPerks();
	player TakeAllWeapons();

	// Give gametype-friendly perks
	player GivePerk("specialty_lightweight");
	player GivePerk("specialty_coldblooded");
	player GivePerk("specialty_marathon");
	player GivePerk("specialty_falldamage");
	player GivePerk("specialty_fastreload");

	gameFlagWait("prematch_done");

	if (!gameFlag("seekers_released")) {
		wait 0.5;

		// Blind player if seekers weren't released yet
		player playLocalSound(game["seekers_blind_sound"]);
		/*for (a = 0; a < RandomIntRange(5, 20); a++) {
			len = RandomFloatRange(0.0, 0.25);
			player VisionSetNakedForPlayer("blacktest", len);
			wait len;
			len = RandomFloatRange(0.0, 0.25);
			player VisionSetNakedForPlayer(getDvar("mapname"), len);
			wait len;
		}
		len = RandomFloatRange(0.0, 0.25);
		player VisionSetNakedForPlayer("blacktest", len);
		*/
		player VisionSetNakedForPlayer("blacktest", 0.1);

		// Freeze, hide
		player FreezeControls(true);
		player Hide();

		gameFlagWait("seekers_released");
		player playLocalSound(game["seekers_released_sound"]);

		// Unfreeze, show
		player FreezeControls(false);
		player VisionSetNakedForPlayer(getDvar("mapname"), 2.5);
		player Show();
	}
	else
	{
		wait 0.5;
	}

	// random primary weapon
	primaryWeapon = level.seekerWeapons[RandomInt(level.seekerWeapons.size)];
	player GiveWeapon(primaryWeapon + "_reflex_mp", RandomInt(9), false);

	// tactical knife
	player GiveWeapon("deserteagle_tactical_mp", 0, false);

	// single semtex
	player GiveWeapon("semtex_mp");
	player SwitchToWeapon(primaryWeapon + "_reflex_mp");

	wait 0.75;

	// ammo
	player GiveMaxAmmo(primaryWeapon + "_reflex_mp");
	player GiveMaxAmmo("deserteagle_tactical_mp");

	player thread enableUnlimitedReload();
	player thread enableWeaponDicing("+actionslot 3");
}

handleHider() {
	logString("handleHider called");

	player = self;

	player endon("disconnect");
	player endon("death");
	level endon("game_ended");

	// Wait for _playerlogic to give the player his default weapons and perks
	player waittill("spawned_player");

	// Go third-person mode
	player SetClientDvar("cg_thirdperson", 1);

	// Take weapons and perks
	player ClearPerks();
	player TakeAllWeapons();

	// Gametype-friendly perks
	player GivePerk("specialty_quieter");
	player GivePerk("specialty_coldblooded");
	player GivePerk("specialty_falldamage");

	gameFlagWait("prematch_done");

	player thread enableThirdPersonSwitch("+actionslot 3", 1);
	player thread enableModelChange("+attack");
	player thread revealWhenGameEnds();

	if (!gameFlag("seekers_released"))
	{
		gameFlagWait("seekers_released");

		/*
		player giveWeapon("deserteagle_tactical_mp", 0, false);
		player switchtoWeapon("deserteagle_tactical_mp");
		*/

		// "Seekers have been released." message on screen
		notifyData = spawnstruct();
		notifyData.titleText = &"HIDEANDSEEK_SEEKERS_RELEASED";
		notifyData.glowColor = (1.0, 0.4, 0.1);
		//notifyData.sound = game["seekers_released_sound"];
		player thread maps\mp\gametypes\_hud_message::notifyMessage(notifyData);
		player PlayLocalSound(game["seekers_released_sound"]);
	}
}

/*
 * Binds a player command to let the player change between first
 * person view and third person view.
 */
enableThirdPersonSwitch(playerCommand, value) {
	logString("enableThirdPersonSwitch called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player notifyOnPlayerCommand("thirdpersonswitch_requested", playerCommand);

	for (;;)
	{
		player waittill("thirdpersonswitch_requested");

		// toggle third person mode
		value = !value;
		player SetClientDvar("cg_thirdperson", value);
		wait 0.5;
	}
}

/*
 * Binds a player command to let the player change into the model they
 * are looking at.
 */
enableModelChange(playerCommand) {
	logString("enableModelChange called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player notifyOnPlayerCommand("modelchange_requested", playerCommand);

	defaultOriginAdd = undefined;

	for (;;)
	{
		player waittill("modelchange_requested");

		if (!IsDefined(defaultOriginAdd))
		{
			defaultOriginAdd = (player GetEye()) - (player GetOrigin());
		}

		if (!IsDefined(player.originalModel))
		{
			player.originalModel = player.model;
		}

		if (!IsDefined(player.originalModelAttachments))
		{
			player.originalModelAttachments = [];
			for (index = 0; index < player GetAttachSize(); index++)
			{
				player.originalModelAttachments[index] = spawnstruct();
				player.originalModelAttachments[index].tag = player GetAttachTagName(index);
				player.originalModelAttachments[index].model = player GetAttachModelName(index);
			}
		}

		// Make a trace to the focused object
		origin = (player GetOrigin()) + defaultOriginAdd;
		trace = BulletTrace(origin, origin + anglestoforward(player getplayerangles()) * 1000000, false, player);
		if (!IsDefined(trace) || !IsDefined(trace["entity"]) || !IsDefined(trace["entity"].model) || !IsDefined(trace["entity"].classname))
		{
			player IPrintLnBold(trace);
			continue;
		}
		if (trace["entity"].classname != "script_model")
		{
			player IPrintLnBold("^3" + trace["entity"].model + " is not a script model.");
			continue;
		}
		if (maps\mp\mods\_modellist::isIgnoredModel(trace["entity"].model))
		{
			player IPrintLnBold("^3You can't change into that!");
			continue;
		}
		displayName = maps\mp\mods\_modellist::getDisplayName(trace["entity"].model);

		healthfrac = player.health / player.maxhealth;
		newhealth = healthfrac * (trace["entity"] GetNormalHealth());

		player SetModel(trace["entity"].model);
		if (IsDefined(game["hiders_changed_sound"]))
			player PlaySound(game["hiders_changed_sound"]);

		player.health = newhealth;
		player.maxhealth = trace["entity"] GetNormalHealth();

		player SetClientDvar("cg_thirdpersonrange", 300);

		// Seems like changing model causes weapons to be reloaded
		player TakeAllWeapons();

		//player SayTeam("(changed into " + displayName + ")");
		player IPrintLnBold("^2You changed into ^4" + displayName);
	}
}

/*
 * Binds a player command to shuffle the weapon.
 */
enableWeaponDicing(playerCommand)
{
	logString("enableWeaponDicing called");

	player = self;

	level endon("game_ended");
	player endon("death");
	player endon("disconnect");

	player notifyOnPlayerCommand("weapondicing_requested", playerCommand);

	for (;;)
	{
		player waittill("weapondicing_requested");

		// random primary weapon
		player TakeWeapon(player GetCurrentWeapon());
		primaryWeapon = level.seekerWeapons[RandomInt(level.seekerWeapons.size)];
		player GiveWeapon(primaryWeapon + "_reflex_mp", RandomInt(9), false);
		player SwitchToWeapon(primaryWeapon + "_reflex_mp");
		player GiveMaxAmmo(primaryWeapon + "_reflex_mp");
	}
}

/*
 * Refills ammo stock after a reload.
 */
enableUnlimitedReload()
{
	logString("enableUnlimitedReload called");

	player = self;

	for (;;)
	{
		player waittill("reload");

		player GiveMaxAmmo(player GetCurrentWeapon());
		//player GiveMaxAmmo(player GetCurrentOffhand());
	}
}

/*
 * Plays a revealing effect when the round ends if this player is
 * still alive.
 */
revealWhenGameEnds()
{
	logString("revealWhenGameEnds called");

	player = self;

	player endon("death");
	player endon("disconnect");

	level waittill("game_ended");

	for (a = 0; a < 5; a++) {
		fxPos = player GetOrigin() + (RandomIntRange(-32, 32), RandomIntRange(-32, 32), RandomIntRange(64, 192));
		PlayFX(game["fx_confetti"], fxPos);
		PlayFX(game["fx_boom"], fxPos);
		wait 0.75;
	}
}